import * as React from 'react';
import { Controller } from '../../components';
import { UqData } from '../../net';
import { UqMan, Action, Sheet, Query, Book, Map, Entity, Tuid, History, Pending, TuidDiv } from '../../uq';
import { CLink } from '../link';
import { CBook, BookUI } from '../book';
import { CSheet, SheetUI } from '../sheet';
import { ActionUI, CAction } from '../action';
import { QueryUI, CQuery, CQuerySelect } from '../query';
import { CTuidMain, TuidUI, CTuidInfo, CTuidSelect, CTuidEdit, CTuidList } from '../tuid';
import { MapUI, CMap } from '../map';
import { CEntity, EntityUI } from '../CVEntity';
import { VUq } from './vUq';
import { CHistory, HistoryUI } from '../history';
import { CPending, PendingUI } from '../pending';
import { CApp } from '../app';
export declare type EntityType = 'sheet' | 'action' | 'tuid' | 'query' | 'book' | 'map' | 'history' | 'pending';
export interface UqUI {
    CTuidMain?: typeof CTuidMain;
    CTuidEdit?: typeof CTuidEdit;
    CTuidList?: typeof CTuidList;
    CTuidSelect?: typeof CTuidSelect;
    CTuidInfo?: typeof CTuidInfo;
    CQuery?: typeof CQuery;
    CQuerySelect?: typeof CQuerySelect;
    CMap?: typeof CMap;
    CAction?: typeof CAction;
    CSheet?: typeof CSheet;
    CBook?: typeof CBook;
    CHistory?: typeof CHistory;
    CPending?: typeof CPending;
    tuid?: {
        [name: string]: TuidUI;
    };
    sheet?: {
        [name: string]: SheetUI;
    };
    action?: {
        [name: string]: ActionUI;
    };
    map?: {
        [name: string]: MapUI;
    };
    query?: {
        [name: string]: QueryUI;
    };
    book?: {
        [name: string]: BookUI;
    };
    history?: {
        [name: string]: HistoryUI;
    };
    pending?: {
        [name: string]: PendingUI;
    };
    res?: any;
}
export declare class CUq extends Controller {
    private ui;
    private tuidURs;
    private CTuidMain;
    private CTuidEdit;
    private CTuidList;
    private CTuidSelect;
    private CTuidInfo;
    private CQuery;
    private CQuerySelect;
    private CMap;
    private CAction;
    private CSheet;
    private CBook;
    private CHistory;
    private CPending;
    constructor(cApp: CApp, uqData: UqData, ui: UqUI);
    protected internalStart(): Promise<void>;
    cApp: CApp;
    res: any;
    uq: UqMan;
    error: string;
    private createBoxId;
    init(): Promise<void>;
    loadEntities(): Promise<string>;
    getQuerySearch(name: string): Promise<Query>;
    getTuidPlaceHolder(tuid: Tuid): any;
    getNone(): any;
    protected isSysVisible: boolean;
    protected isVisible(entity: Entity): boolean;
    navSheet(sheetTypeId: number, sheetId: number): Promise<void>;
    sheet(entityName: string): Sheet;
    action(entityName: string): Action;
    query(entityName: string): Query;
    book(entityName: string): Book;
    map(entityName: string): Map;
    history(entityName: string): History;
    pending(entityName: string): Pending;
    tuid(entityName: string): Tuid;
    tuidDiv(entityName: string, divName: string): TuidDiv;
    cSheetFromName(entityName: string): CSheet;
    cActionFromName(entityName: string): CAction;
    cQueryFromName(entityName: string): CQuery;
    cBookFromName(entityName: string): CBook;
    cMapFromName(entityName: string): CMap;
    cHistoryFromName(entityName: string): CHistory;
    cPendingFromName(entityName: string): CPending;
    cTuidMainFromName(entityName: string): CTuidMain;
    cTuidEditFromName(entityName: string): CTuidEdit;
    cTuidInfoFromName(entityName: string): CTuidInfo;
    cTuidSelectFromName(entityName: string): CTuidSelect;
    cFromName(entityType: EntityType, entityName: string): CEntity<Entity, EntityUI>;
    linkFromName(entityType: EntityType, entityName: string): CLink;
    private getUI;
    link(cEntity: CEntity<Entity, EntityUI>): CLink;
    get tuidLinks(): CLink[];
    cTuidMain(tuid: Tuid): CTuidMain;
    cTuidEdit(tuid: Tuid): CTuidEdit;
    cTuidList(tuid: Tuid): CTuidList;
    cTuidSelect(tuid: Tuid): CTuidSelect;
    cTuidInfo(tuid: Tuid): CTuidInfo;
    cSheet(sheet: Sheet): CSheet;
    get sheetLinks(): CLink[];
    cAction(action: Action): CAction;
    get actionLinks(): CLink[];
    cQuery(query: Query): CQuery;
    cQuerySelect(queryName: string): CQuerySelect;
    get queryLinks(): CLink[];
    cBook(book: Book): CBook;
    get bookLinks(): CLink[];
    cHistory(history: History): CHistory;
    get historyLinks(): CLink[];
    cPending(pending: Pending): CPending;
    get pendingLinks(): CLink[];
    cMap(map: Map): CMap;
    get mapLinks(): CLink[];
    getTuidContent(tuid: Tuid): React.StatelessComponent<any>;
    getTuidDivContent(tuidDiv: TuidDiv): React.StatelessComponent<any>;
    showTuid(tuid: Tuid, id: number): Promise<void>;
    showTuidDiv(tuid: TuidDiv, id: number): Promise<void>;
    protected get VUq(): typeof VUq;
    render(): JSX.Element;
}
